#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import unicode_literals

from django.test import TestCase

from multigtfs.models.fields import Seconds, SecondsField


class SecondsTest(TestCase):

    def test_string(self):
        self.assertEqual('00:00:00', str(Seconds(0)))
        self.assertEqual('24:00:00', str(Seconds(24 * 60 * 60)))
        self.assertEqual('12:34:56', str(Seconds(45296)))

    def test_negative_seconds_raises(self):
        self.assertRaises(ValueError, Seconds, -1)

    def test_from_hms(self):
        standard = Seconds(45296)
        from_hms = Seconds.from_hms(hours=12, minutes=34, seconds=56)
        self.assertEqual(standard, from_hms)

    def test_unequal_int(self):
        seconds = Seconds(1000)
        self.assertNotEqual(seconds, 1000)

    def test_comparison(self):
        one_minute = Seconds(60)
        one_hour = Seconds(3600)
        one_hour2 = Seconds(3600)
        self.assertTrue(one_hour > one_minute)
        self.assertFalse(one_hour > one_hour2)
        self.assertTrue(one_hour >= one_hour2)
        self.assertTrue(one_hour == one_hour2)
        self.assertFalse(one_hour != one_hour2)
        self.assertFalse(one_hour < one_hour2)
        self.assertTrue(one_hour <= one_hour2)


class SecondsFieldTest(TestCase):

    def setUp(self):
        self.f = SecondsField()

    def test_to_python_int(self):
        self.assertEqual(Seconds(400), self.f.to_python(400))

    def test_to_python_Seconds(self):
        self.assertEqual(Seconds(500), self.f.to_python(Seconds(500)))

    def test_to_python_seconds_string(self):
        self.assertEqual(Seconds(500), self.f.to_python('500'))

    def test_to_python_hm_string(self):
        self.assertEqual(Seconds(3660), self.f.to_python('01:01'))

    def test_to_python_hms_string(self):
        self.assertEqual(Seconds(3661), self.f.to_python('01:01:01'))

    def test_to_python_too_many_colons(self):
        self.assertRaises(ValueError, self.f.to_python, '01:01:01:01')

    def test_to_python_none(self):
        self.assertIsNone(self.f.to_python(None))

    def test_to_python_empty_string(self):
        self.assertIsNone(self.f.to_python(''))

    def test_prep_db_value_Seconds(self):
        self.assertEqual(500, self.f.get_prep_value(Seconds(500)))

    def test_prep_db_value_None(self):
        self.assertIsNone(self.f.get_prep_value(None))



# Copy to t_overrides.py, make changes

import os


def update(old_config):
    config = old_config.copy()

    if os.environ.get('MULTIGTFS_TEST_POSTGIS', False):
        # Run tests under PostgreSQL / PostGIS
        # Requires psycopg2:
        #    pip install psycopg2
        # Requires the PostgreSQL role with SUPERUSER and CREATEDB permission
        #    createuser multigtfs --superuser --createdb --pwprompt
        # Requires the PostgreSQL database w/ PostGIS extensions:
        #    createdb multigtfs --owner=multigtfs
        #    psql -d multigtfs -c "CREATE EXTENSION postgis"
        #    psql -d multigtfs -c "CREATE EXTENSION postgis_topology"
        # Running the test creates a new database 'test_multigtfs'
        # Because superuser permissions are needed, it is recommended that you
        #  only install on a local database, and change configuration items
        #  below.
        config['DATABASE_ENGINE'] = 'django.contrib.gis.db.backends.postgis'
        config['DATABASES'] = {
            'default': {
                'ENGINE': 'django.contrib.gis.db.backends.postgis',
                'NAME': 'multigtfs',
                'USER': 'multigtfs',
                'PASSWORD': 'multigtfs',
                'HOST': 'localhost',
                'PORT': '5432'}}
    return config



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import unicode_literals

from django.test import TestCase
from django.utils.six import StringIO

from multigtfs.models import Feed, Stop, Transfer


class TransferTest(TestCase):
    def setUp(self):
        self.feed = Feed.objects.create()
        self.stop1 = Stop.objects.create(
            feed=self.feed, stop_id='STOP1', point="POINT(-117.1 36.4)")
        self.stop2 = Stop.objects.create(
            feed=self.feed, stop_id='STOP2', point="POINT(-117.2 36.5)")

    def test_string(self):
        transfer = Transfer.objects.create(
            from_stop=self.stop1, to_stop=self.stop2)
        self.assertEqual(str(transfer), '%d-STOP1-STOP2' % self.feed.id)

    def test_import_transfers_txt_minimal(self):
        transfers_txt = StringIO("""\
from_stop_id,to_stop_id
STOP1,STOP2
""")
        Transfer.import_txt(transfers_txt, self.feed)
        transfer = Transfer.objects.get()
        self.assertEqual(transfer.from_stop, self.stop1)
        self.assertEqual(transfer.to_stop, self.stop2)
        self.assertEqual(transfer.transfer_type, 0)
        self.assertEqual(transfer.min_transfer_time, None)

    def test_import_transfers_txt_duplicate(self):
        transfers_txt = StringIO("""\
from_stop_id,to_stop_id
STOP1,STOP2
STOP1,STOP2
""")
        Transfer.import_txt(transfers_txt, self.feed)
        transfer = Transfer.objects.get()  # Just one
        self.assertEqual(transfer.from_stop, self.stop1)
        self.assertEqual(transfer.to_stop, self.stop2)

    def test_import_transfers_txt_maximal(self):
        transfers_txt = StringIO("""\
from_stop_id,to_stop_id,transfer_type,min_transfer_time
STOP1,STOP2,2,5
""")
        Transfer.import_txt(transfers_txt, self.feed)
        transfer = Transfer.objects.get()
        self.assertEqual(transfer.from_stop, self.stop1)
        self.assertEqual(transfer.to_stop, self.stop2)
        self.assertEqual(transfer.transfer_type, 2)
        self.assertEqual(transfer.min_transfer_time, 5)

    def test_import_transfers_txt_omitted(self):
        transfers_txt = StringIO("""\
from_stop_id,to_stop_id,transfer_type,min_transfer_time
STOP1,STOP2,,
""")
        Transfer.import_txt(transfers_txt, self.feed)
        transfer = Transfer.objects.get()
        self.assertEqual(transfer.from_stop, self.stop1)
        self.assertEqual(transfer.to_stop, self.stop2)
        self.assertEqual(transfer.transfer_type, 0)
        self.assertEqual(transfer.min_transfer_time, None)

    def test_export_transfers_empty(self):
        transfers_txt = Transfer.export_txt(self.feed)
        self.assertFalse(transfers_txt)

    def test_export_transfers_minimal(self):
        Transfer.objects.create(
            from_stop=self.stop1, to_stop=self.stop2)
        transfers_txt = Transfer.export_txt(self.feed)
        self.assertEqual(transfers_txt, """\
from_stop_id,to_stop_id,transfer_type
STOP1,STOP2,0
""")

    def test_export_transfers_maximal(self):
        Transfer.objects.create(
            from_stop=self.stop1, to_stop=self.stop2, transfer_type=2,
            min_transfer_time=5)
        transfers_txt = Transfer.export_txt(self.feed)
        self.assertEqual(transfers_txt, """\
from_stop_id,to_stop_id,transfer_type,min_transfer_time
STOP1,STOP2,2,5
""")

    def test_export_transfers_two(self):
        Transfer.objects.create(
            from_stop=self.stop2, to_stop=self.stop1)
        Transfer.objects.create(
            from_stop=self.stop1, to_stop=self.stop2, transfer_type=2,
            min_transfer_time=5)
        transfers_txt = Transfer.export_txt(self.feed)
        self.assertEqual(transfers_txt, """\
from_stop_id,to_stop_id,transfer_type,min_transfer_time
STOP1,STOP2,2,5
STOP2,STOP1,0,
""")



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import unicode_literals
from datetime import date

from django.test import TestCase
from django.utils.six import StringIO

from multigtfs.models import Feed, FeedInfo


class FeedInfoTest(TestCase):
    def setUp(self):
        self.feed = Feed.objects.create()

    def test_string(self):
        feed_info = FeedInfo.objects.create(
            feed=self.feed, publisher_name='PTEST',
            publisher_url='http://example.com', lang='en')
        self.assertEqual(str(feed_info), '%d-PTEST' % self.feed.id)

    def test_import_feed_info_txt_maximal(self):
        feed_info_txt = StringIO("""\
feed_publisher_name,feed_publisher_url,feed_lang,feed_start_date,\
feed_end_date,feed_version
PTEST,http://example.com,en,20120414,20121231,FOO1
""")
        FeedInfo.import_txt(feed_info_txt, feed=self.feed)
        feed_info = FeedInfo.objects.get()
        self.assertEqual(feed_info.publisher_name, 'PTEST')
        self.assertEqual(feed_info.publisher_url, 'http://example.com')
        self.assertEqual(feed_info.lang, 'en')
        self.assertEqual(feed_info.start_date, date(2012, 4, 14))
        self.assertEqual(feed_info.end_date, date(2012, 12, 31))
        self.assertEqual(feed_info.version, 'FOO1')

    def test_import_feed_info_txt_minimal(self):
        feed_info_txt = StringIO("""\
feed_publisher_name,feed_publisher_url
PTEST,http://example.com
""")
        FeedInfo.import_txt(feed_info_txt, feed=self.feed)
        feed_info = FeedInfo.objects.get()
        self.assertEqual(feed_info.publisher_name, 'PTEST')
        self.assertEqual(feed_info.publisher_url, 'http://example.com')
        self.assertEqual(feed_info.lang, '')
        self.assertEqual(feed_info.start_date, None)
        self.assertEqual(feed_info.end_date, None)
        self.assertEqual(feed_info.version, '')

    def test_import_feed_info_txt_duplicate(self):
        feed_info_txt = StringIO("""\
feed_publisher_name,feed_publisher_url
PTEST,http://example.com
PTEST,http://www.example.com
""")
        FeedInfo.import_txt(feed_info_txt, feed=self.feed)
        feed_info = FeedInfo.objects.get()  # Just one
        self.assertEqual(feed_info.publisher_name, 'PTEST')
        self.assertEqual(feed_info.publisher_url, 'http://example.com')

    def test_export_feed_info_txt_empty(self):
        feed_info_txt = FeedInfo.export_txt(self.feed)
        self.assertFalse(feed_info_txt)

    def test_export_feed_info_txt_minimal(self):
        FeedInfo.objects.create(
            feed=self.feed, publisher_name='PTEST',
            publisher_url='http://example.com', lang='en')
        feed_info_txt = FeedInfo.export_txt(self.feed)
        self.assertEqual(feed_info_txt, """\
feed_publisher_name,feed_publisher_url,feed_lang
PTEST,http://example.com,en
""")

    def test_export_feed_info_txt_maximal(self):
        FeedInfo.objects.create(
            feed=self.feed, publisher_name='PTEST',
            publisher_url='http://example.com', lang='en',
            start_date=date(2012, 9, 2), end_date=date(2013, 1, 1),
            version='BAR1')
        feed_info_txt = FeedInfo.export_txt(self.feed)
        self.assertEqual(feed_info_txt, """\
feed_publisher_name,feed_publisher_url,feed_lang,feed_start_date,\
feed_end_date,feed_version
PTEST,http://example.com,en,20120902,20130101,BAR1
""")



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals
import warnings

from django.contrib.gis.geos import LineString
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.encoding import python_2_unicode_compatible
from jsonfield import JSONField

from multigtfs.models.base import models, Base


@python_2_unicode_compatible
class Shape(Base):
    """The path the vehicle takes along the route.

    Implements shapes.txt."""
    feed = models.ForeignKey('Feed')
    shape_id = models.CharField(
        max_length=255, db_index=True,
        help_text="Unique identifier for a shape.")
    geometry = models.LineStringField(
        null=True, blank=True,
        help_text='Geometry cache of ShapePoints')

    def __str__(self):
        return "%d-%s" % (self.feed.id, self.shape_id)

    def update_geometry(self, update_parent=True):
        """Update the geometry from the related ShapePoints"""
        original = self.geometry
        points = self.points.order_by(
            'sequence').values_list('point', flat=True)
        if len(points) > 1:
            self.geometry = LineString([pt.coords for pt in points])
            if self.geometry != original:
                self.save()
                if update_parent:
                    for trip in self.trip_set.all():
                        trip.update_geometry()

    class Meta:
        db_table = 'shape'
        app_label = 'multigtfs'

    _rel_to_feed = 'feed'


@python_2_unicode_compatible
class ShapePoint(Base):
    """A point along the shape"""
    shape = models.ForeignKey('Shape', related_name='points')
    point = models.PointField(
        help_text='WGS 84 latitude/longitude of shape point')
    sequence = models.IntegerField()
    traveled = models.FloatField(
        null=True, blank=True,
        help_text='Distance of point from start of shape')
    extra_data = JSONField(default={}, blank=True, null=True)

    def __str__(self):
        return "%s-%d" % (self.shape, self.sequence)

    def getlon(self):
        return self.point[0] if self.point else 0.0

    def setlon(self, value):
        if self.point:
            self.point[0] = value
        else:
            self.point = "POINT(%s 0)" % value

    lon = property(getlon, setlon, doc="WGS 84 longitude of shape point")

    def getlat(self):
        return self.point[1] if self.point else 0.0

    def setlat(self, value):
        if self.point:
            self.point[1] = value
        else:
            self.point = "POINT(0 %s)" % value

    lat = property(getlat, setlat, doc="WGS 84 latitude of shape point")

    def __init__(self, *args, **kwargs):
        """Initialize a ShapePoint

        If the legacy lat and lon params are used, then warn and initialize
        the point from them.
        """
        lat = kwargs.pop('lat', None)
        lon = kwargs.pop('lon', None)
        if lat is not None or lon is not None:
            assert kwargs.get('point') is None
            msg = "Setting ShapePoint location with lat and lon is deprecated"
            warnings.warn(msg, DeprecationWarning)
            kwargs['point'] = "POINT(%s %s)" % (lon or 0.0, lat or 0.0)

        super(ShapePoint, self).__init__(*args, **kwargs)

    class Meta:
        db_table = 'shape_point'
        app_label = 'multigtfs'

    _column_map = (
        ('shape_id', 'shape__shape_id'),
        ('shape_pt_lat', 'point[1]'),
        ('shape_pt_lon', 'point[0]'),
        ('shape_pt_sequence', 'sequence'),
        ('shape_dist_traveled', 'traveled')
    )
    _filename = 'shapes.txt'
    _rel_to_feed = 'shape__feed'
    _sort_order = ('shape__shape_id', 'sequence')
    _unique_fields = ('shape_id', 'shape_pt_sequence')


@receiver(post_save, sender=ShapePoint, dispatch_uid="post_save_shapepoint")
def post_save_shapepoint(sender, instance, **kwargs):
    '''Update related objects when the ShapePoint is updated'''
    instance.shape.update_geometry()



from django.conf.urls import url
from django.views.generic import DetailView, ListView
from multigtfs.models import (
    Agency, Block, Fare, FareRule, Feed, FeedInfo, Route, Service, ServiceDate,
    Shape, ShapePoint, Stop, StopTime, Trip, Zone)

from exploreapp.views import (
    ByFeedListView, FareRuleByFareListView, FareRuleByRouteListView,
    FrequencyByTripListView, ServiceDateByServiceListView,
    ShapePointByShapeListView, StopTimeByStopListView, StopTimeByTripListView,
    TripByBlockListView, TripByRouteListView, TripByServiceListView,
    TripByShapeListView)


urlpatterns = [
    url(r'feed/$', ListView.as_view(model=Feed), name='feed_list'),
    url(r'feed/(?P<pk>\d+)/$', DetailView.as_view(model=Feed),
        name='feed_detail'),
    url(r'feed/(?P<feed_id>\d+)/agency/$',
        ByFeedListView.as_view(model=Agency),
        name='agency_list'),
    url(r'feed/(?P<feed_id>\d+)/agency/(?P<pk>\d+)/$',
        DetailView.as_view(model=Agency), name='agency_detail'),
    url(r'feed/(?P<feed_id>\d+)/stop/$',
        ByFeedListView.as_view(model=Stop),
        name='stop_list'),
    url(r'feed/(?P<feed_id>\d+)/stop/(?P<pk>\d+)/$',
        DetailView.as_view(model=Stop), name='stop_detail'),
    url(r'feed/(?P<feed_id>\d+)/stop/(?P<stop_id>\d+)/stoptime/$',
        StopTimeByStopListView.as_view(), name='stoptime_by_stop_list'),
    url(r'feed/(?P<feed_id>\d+)/stoptime/(?P<pk>\d+)/$',
        DetailView.as_view(model=StopTime), name='stoptime_detail'),
    url(r'feed/(?P<feed_id>\d+)/trip/(?P<pk>\d+)/$',
        DetailView.as_view(model=Trip), name='trip_detail'),
    url(r'feed/(?P<feed_id>\d+)/trip/(?P<trip_id>\d+)/frequency/$',
        FrequencyByTripListView.as_view(), name='frequency_by_trip_list'),
    url(r'feed/(?P<feed_id>\d+)/trip/(?P<trip_id>\d+)/stoptime/$',
        StopTimeByTripListView.as_view(), name='stoptime_by_trip_list'),
    url(r'feed/(?P<feed_id>\d+)/route/(?P<route_id>\d+)/farerule/$',
        FareRuleByRouteListView.as_view(), name='farerule_by_route_list'),
    url(r'feed/(?P<feed_id>\d+)/route/(?P<route_id>\d+)/trip/$',
        TripByRouteListView.as_view(), name='trip_by_route_list'),
    url(r'feed/(?P<feed_id>\d+)/route/$',
        ByFeedListView.as_view(model=Route),
        name='route_list'),
    url(r'feed/(?P<feed_id>\d+)/route/(?P<pk>\d+)/$',
        DetailView.as_view(model=Route), name='route_detail'),
    url(r'feed/(?P<feed_id>\d+)/service/$',
        ByFeedListView.as_view(model=Service),
        name='service_list'),
    url(r'feed/(?P<feed_id>\d+)/service/(?P<pk>\d+)/$',
        DetailView.as_view(model=Service), name='service_detail'),
    url(r'feed/(?P<feed_id>\d+)/service/(?P<service_id>\d+)/servicedate/$',
        ServiceDateByServiceListView.as_view(),
        name='servicedate_by_service_list'),
    url(r'feed/(?P<feed_id>\d+)/service/(?P<service_id>\d+)/trip/$',
        TripByServiceListView.as_view(), name='trip_by_service_list'),
    url(r'feed/(?P<feed_id>\d+)/servicedate/(?P<pk>\d+)/$',
        DetailView.as_view(model=ServiceDate), name='servicedate_detail'),
    url(r'feed/(?P<feed_id>\d+)/shape/$',
        ByFeedListView.as_view(model=Shape),
        name='shape_list'),
    url(r'feed/(?P<feed_id>\d+)/shape/(?P<pk>\d+)/$',
        DetailView.as_view(model=Shape), name='shape_detail'),
    url(r'feed/(?P<feed_id>\d+)/shape/(?P<shape_id>\d+)/shapepoint/$',
        ShapePointByShapeListView.as_view(), name='shapepoint_by_shape_list'),
    url(r'feed/(?P<feed_id>\d+)/shape/(?P<shape_id>\d+)/trip/$',
        TripByShapeListView.as_view(), name='trip_by_shape_list'),
    url(r'feed/(?P<feed_id>\d+)/shapepoint/(?P<pk>\d+)$',
        DetailView.as_view(model=ShapePoint), name='shapepoint_detail'),
    url(r'feed/(?P<feed_id>\d+)/fare/$',
        ByFeedListView.as_view(model=Fare),
        name='fare_list'),
    url(r'feed/(?P<feed_id>\d+)/fare/(?P<pk>\d+)/$',
        DetailView.as_view(model=Fare), name='fare_detail'),
    url(r'feed/(?P<feed_id>\d+)/fare/(?P<fare_id>\d+)/farerule$',
        FareRuleByFareListView.as_view(), name='farerule_by_fare_list'),
    url(r'feed/(?P<feed_id>\d+)/farerule/(?P<pk>\d+)/$',
        DetailView.as_view(model=FareRule), name='farerule_detail'),
    url(r'feed/(?P<feed_id>\d+)/feedinfo/$',
        ByFeedListView.as_view(model=FeedInfo),
        name='feedinfo_list'),
    url(r'feed/(?P<feed_id>\d+)/feedinfo/(?P<pk>\d+)/$',
        DetailView.as_view(model=FeedInfo), name='feedinfo_detail'),
    url(r'feed/(?P<feed_id>\d+)/zone/$',
        ByFeedListView.as_view(model=Zone),
        name='zone_list'),
    url(r'feed/(?P<feed_id>\d+)/zone/(?P<pk>\d+)/$',
        DetailView.as_view(model=Zone), name='zone_detail'),
    url(r'feed/(?P<feed_id>\d+)/block/$',
        ByFeedListView.as_view(model=Block),
        name='block_list'),
    url(r'feed/(?P<feed_id>\d+)/block/(?P<pk>\d+)/$',
        DetailView.as_view(model=Block), name='block_detail'),
    url(r'feed/(?P<feed_id>\d+)/block/(?P<block_id>\d+)/trip/$',
        TripByBlockListView.as_view(), name='trip_by_block_list'),
]



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import unicode_literals
from datetime import date

from django.test import TestCase
from django.utils.six import StringIO

from multigtfs.models import Feed, Service


class ServiceTest(TestCase):
    def setUp(self):
        self.feed = Feed.objects.create()

    def test_string(self):
        service = Service.objects.create(
            feed=self.feed, service_id='S1', start_date=date(2011, 4, 14),
            end_date=date(2011, 12, 31))
        self.assertEqual(str(service), '%d-S1' % self.feed.id)

    def test_import_calendar_txt(self):
        calendar_txt = StringIO("""\
service_id,monday,tuesday,wednesday,thursday,friday,saturday,sunday,\
start_date,end_date
W,1,0,1,0,1,0,1,20120414,20121231
""")
        Service.import_txt(calendar_txt, self.feed)
        service = Service.objects.get()
        self.assertEqual(service.feed, self.feed)
        self.assertEqual(service.service_id, 'W')
        self.assertTrue(service.monday)
        self.assertFalse(service.tuesday)
        self.assertTrue(service.wednesday)
        self.assertFalse(service.thursday)
        self.assertTrue(service.friday)
        self.assertFalse(service.saturday)
        self.assertTrue(service.sunday)
        self.assertEqual(service.start_date, date(2012, 4, 14))
        self.assertEqual(service.end_date, date(2012, 12, 31))

    def test_import_calendar_duplicate(self):
        calendar_txt = StringIO("""\
service_id,monday,tuesday,wednesday,thursday,friday,saturday,sunday,\
start_date,end_date
W,1,0,1,0,1,0,1,20120414,20121231
W,0,1,0,1,0,1,0,20120414,20121231
""")
        Service.import_txt(calendar_txt, self.feed)
        service = Service.objects.get()  # Only one
        self.assertEqual(service.feed, self.feed)
        self.assertEqual(service.service_id, 'W')

    def test_export_calendar_txt_none(self):
        calendar_txt = Service.export_txt(self.feed)
        self.assertFalse(calendar_txt)

    def test_export_calendar_txt(self):
        Service.objects.create(
            feed=self.feed, service_id='W', monday=True, tuesday=False,
            wednesday=True, thursday=False, friday=True, saturday=False,
            sunday=True, start_date=date(2012, 7, 17),
            end_date=date(2013, 7, 17))
        calendar_txt = Service.export_txt(self.feed)
        self.assertEqual(calendar_txt, """\
service_id,monday,tuesday,wednesday,thursday,friday,saturday,sunday,\
start_date,end_date
W,1,0,1,0,1,0,1,20120717,20130717
""")



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from django.utils.encoding import python_2_unicode_compatible
from jsonfield import JSONField

from multigtfs.models.base import models, Base


@python_2_unicode_compatible
class FeedInfo(Base):
    """Information about the feed

    Implements feed_info.txt in the GTFS feed.
    """
    feed = models.ForeignKey('Feed')
    publisher_name = models.CharField(
        max_length=255,
        help_text="Full name of organization that publishes the feed.")
    publisher_url = models.URLField(
        help_text="URL of the feed publisher's organization.")
    lang = models.CharField(
        "language",
        max_length=20,
        help_text="IETF BCP 47 language code for text in field.")
    start_date = models.DateField(
        null=True, blank=True,
        help_text="Date that feed starts providing reliable data.")
    end_date = models.DateField(
        null=True, blank=True,
        help_text="Date that feed stops providing reliable data.")
    version = models.CharField(
        max_length=255, blank=True,
        help_text="Version of feed.")
    extra_data = JSONField(default={}, blank=True, null=True)

    def __str__(self):
        return '%s-%s' % (self.feed.id, self.publisher_name)

    class Meta:
        db_table = 'feed_info'
        app_label = 'multigtfs'
        verbose_name_plural = "feed info"

    _column_map = (
        ('feed_publisher_name', 'publisher_name'),
        ('feed_publisher_url', 'publisher_url'),
        ('feed_lang', 'lang'),
        ('feed_start_date', 'start_date'),
        ('feed_end_date', 'end_date'),
        ('feed_version', 'version')
    )
    _filename = 'feed_info.txt'
    _unique_fields = ('feed_publisher_name',)



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from django.utils.encoding import python_2_unicode_compatible
from jsonfield import JSONField

from multigtfs.models.base import models, Base


@python_2_unicode_compatible
class FareRule(Base):
    """Associate a Fare with a Route and/or Zones"""
    fare = models.ForeignKey('Fare')
    route = models.ForeignKey(
        'Route', null=True, blank=True,
        help_text="Fare class is valid for this route.")
    origin = models.ForeignKey(
        'Zone', null=True, blank=True,
        related_name='fare_origins',
        help_text="Fare class is valid for travel originating in this zone.")
    destination = models.ForeignKey(
        'Zone', null=True, blank=True,
        related_name='fare_destinations',
        help_text="Fare class is valid for travel ending in this zone.")
    contains = models.ForeignKey(
        'Zone', null=True, blank=True,
        related_name='fare_contains',
        help_text="Fare class is valid for travel withing this zone.")
    extra_data = JSONField(default={}, blank=True, null=True)

    def __str__(self):
        u = "%d-%s" % (self.fare.feed.id, self.fare.fare_id)
        if self.route:
            u += '-%s' % self.route.route_id
        return u

    class Meta:
        db_table = 'fare_rules'
        app_label = 'multigtfs'

    # For Base import/export
    _column_map = (
        ('fare_id', 'fare__fare_id'),
        ('route_id', 'route__route_id'),
        ('origin_id', 'origin__zone_id'),
        ('destination_id', 'destination__zone_id'),
        ('contains_id', 'contains__zone_id')
    )
    _filename = 'fare_rules.txt'
    _rel_to_feed = 'fare__feed'
    _sort_order = ('route__route_id', 'fare__fare_id')
    _unique_fields = (
        'fare_id', 'route_id', 'origin_id', 'destination_id', 'contains_id')



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals
from logging import getLogger
import warnings

from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.encoding import python_2_unicode_compatible
from django.utils.six import StringIO
from jsonfield import JSONField

from multigtfs.models.base import models, Base


logger = getLogger(__name__)


@python_2_unicode_compatible
class Stop(Base):
    """A stop or station

    Maps to stops.txt in the GTFS feed.
    """
    feed = models.ForeignKey('Feed')
    stop_id = models.CharField(
        max_length=255, db_index=True,
        help_text="Unique identifier for a stop or station.")
    code = models.CharField(
        max_length=255, blank=True,
        help_text="Uniquer identifier (short text or number) for passengers.")
    name = models.CharField(
        max_length=255,
        help_text="Name of stop in local vernacular.")
    desc = models.CharField(
        "description",
        max_length=255, blank=True,
        help_text='Description of a stop.')
    point = models.PointField(
        help_text='WGS 84 latitude/longitude of stop or station')
    zone = models.ForeignKey(
        'Zone', null=True, blank=True,
        help_text="Fare zone for a stop ID.")
    url = models.URLField(
        blank=True, help_text="URL for the stop")
    location_type = models.CharField(
        max_length=1, blank=True, choices=(('0', 'Stop'), ('1', 'Station')),
        help_text="Is this a stop or station?")
    parent_station = models.ForeignKey(
        'Stop', null=True, blank=True,
        help_text="The station associated with the stop")
    timezone = models.CharField(
        max_length=255, blank=True,
        help_text="Timezone of the stop")
    wheelchair_boarding = models.CharField(
        max_length=1, blank=True,
        choices=(
            ('0', 'No information'),
            ('1', 'Some wheelchair boarding'),
            ('2', 'No wheelchair boarding')),
        help_text='Is wheelchair boarding possible?')
    extra_data = JSONField(default={}, blank=True, null=True)

    def __str__(self):
        return "%d-%s" % (self.feed_id, self.stop_id)

    def getlon(self):
        return self.point[0] if self.point else 0.0

    def setlon(self, value):
        if self.point:
            self.point[0] = value
        else:
            self.point = "POINT(%s 0)" % value

    lon = property(getlon, setlon, doc="WGS 84 longitude of stop or station")

    def getlat(self):
        return self.point[1] if self.point else 0.0

    def setlat(self, value):
        if self.point:
            self.point[1] = value
        else:
            self.point = "POINT(0 %s)" % value

    lat = property(getlat, setlat, doc="WGS 84 latitude of stop or station")

    def __init__(self, *args, **kwargs):
        lat = kwargs.pop('lat', None)
        lon = kwargs.pop('lon', None)
        if lat is not None or lon is not None:
            assert kwargs.get('point') is None
            msg = "Setting Stop location with lat and lon is deprecated"
            warnings.warn(msg, DeprecationWarning)
            kwargs['point'] = "POINT(%s %s)" % (lon or 0.0, lat or 0.0)
        super(Stop, self).__init__(*args, **kwargs)

    class Meta:
        db_table = 'stop'
        app_label = 'multigtfs'

    _column_map = (
        ('stop_id', 'stop_id'),
        ('stop_code', 'code'),
        ('stop_name', 'name'),
        ('stop_desc', 'desc'),
        ('stop_lat', 'point[1]'),
        ('stop_lon', 'point[0]'),
        ('zone_id', 'zone__zone_id'),
        ('stop_url', 'url'),
        ('location_type', 'location_type'),
        ('parent_station', 'parent_station__stop_id'),
        ('stop_timezone', 'timezone'),
        ('wheelchair_boarding', 'wheelchair_boarding'),
    )
    _filename = 'stops.txt'
    _unique_fields = ('stop_id',)

    @classmethod
    def import_txt(cls, txt_file, feed):
        '''Import from a stops.txt file

        Stations need to be imported before stops
        '''

        txt = txt_file.read()

        def is_station(pairs):
            '''Does the row represent a station?'''
            for name, val in pairs:
                if name == 'location_type':
                    return val == '1'
            return False

        logger.info("Importing station stops")
        stations = super(Stop, cls).import_txt(StringIO(txt), feed, is_station)
        logger.info("Imported %d station stops", stations)

        def is_stop(pairs):
            '''Does the row represent a stop?'''
            for name, val in pairs:
                if name == 'location_type':
                    return val != '1'
            return True

        logger.info("Importing non-station stops")
        stops = super(Stop, cls).import_txt(StringIO(txt), feed, is_stop)
        logger.info("Imported %d non-station stops", stops)
        return stations + stops


@receiver(post_save, sender=Stop, dispatch_uid="post_save_stop")
def post_save_stop(sender, instance, **kwargs):
    '''Update related objects when the Stop is updated'''
    from multigtfs.models.trip import Trip
    trip_ids = instance.stoptime_set.filter(
        trip__shape=None).values_list('trip_id', flat=True).distinct()
    for trip in Trip.objects.filter(id__in=trip_ids):
        trip.update_geometry()



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import unicode_literals

from django.contrib.gis.geos import MultiLineString
from django.test import TestCase
from django.utils.six import StringIO

from multigtfs.compat import bom_prefix_csv, force_utf8
from multigtfs.models import Feed, Route, Stop, StopTime, Trip, Zone


class StopTest(TestCase):
    def setUp(self):
        self.feed = Feed.objects.create()

    def test_string(self):
        stop = Stop.objects.create(
            feed=self.feed, stop_id='STEST',
            point="POINT(-117.133162 36.425288)")
        self.assertEqual(str(stop), '%d-STEST' % self.feed.id)

    def test_legacy_lat_long(self):
        stop1 = Stop(feed=self.feed, stop_id='STOP1')
        stop1.lat = 36.425288
        stop1.lon = -117.133162
        stop1.save()
        stop2 = Stop(feed=self.feed, stop_id='STOP2')
        stop2.lon = -117.14
        stop2.lat = 36.43
        stop2.save()
        self.assertEqual(stop1.point.coords, (-117.133162, 36.425288))
        self.assertEqual(stop1.lat, 36.425288)
        self.assertEqual(stop1.lon, -117.133162)
        self.assertEqual(stop2.point.coords, (-117.14, 36.43))
        self.assertEqual(stop2.lat, 36.43)
        self.assertEqual(stop2.lon, -117.14)

    def test_legacy_create_with_lat_lon(self):
        stop = Stop.objects.create(
            feed=self.feed, stop_id='STEST',
            lat="36.425288", lon="-117.133162")
        self.assertEqual(stop.point.coords, (-117.133162, 36.425288))

    def test_import_stops_txt_none(self):
        stops_txt = StringIO("""\
stop_id,stop_name,stop_desc,stop_lat,stop_lon
""")
        Stop.import_txt(stops_txt, self.feed)
        self.assertFalse(Stop.objects.exists())

    def test_import_stops_txt_minimal(self):
        stops_txt = StringIO("""\
stop_id,stop_name,stop_desc,stop_lat,stop_lon
FUR_CREEK_RES,Furnace Creek Resort (Demo),,36.425288,-117.133162
""")
        Stop.import_txt(stops_txt, self.feed)
        stop = Stop.objects.get()
        self.assertEqual(stop.feed, self.feed)
        self.assertEqual(stop.stop_id, 'FUR_CREEK_RES')
        self.assertEqual(stop.code, '')
        self.assertEqual(stop.name, 'Furnace Creek Resort (Demo)')
        self.assertEqual(stop.desc, '')
        self.assertEqual(str(stop.lat), '36.425288')
        self.assertEqual(str(stop.lon), '-117.133162')
        self.assertEqual(stop.zone, None)
        self.assertEqual(stop.url, '')
        self.assertEqual(stop.location_type, '')
        self.assertEqual(stop.parent_station, None)
        self.assertEqual(stop.timezone, '')
        self.assertEqual(stop.wheelchair_boarding, '')

    def test_import_stops_txt_extra_columns(self):
        stops_txt = StringIO("""\
stop_id,stop_name,stop_desc,stop_lat,stop_lon,platform_code
FUR_CREEK_RES,Furnace Creek Resort (Demo),,36.425288,-117.133162,7
""")
        Stop.import_txt(stops_txt, self.feed)
        stop = Stop.objects.get()
        self.assertEqual(stop.feed, self.feed)
        self.assertEqual(stop.stop_id, 'FUR_CREEK_RES')
        self.assertEqual(stop.code, '')
        self.assertEqual(stop.name, 'Furnace Creek Resort (Demo)')
        self.assertEqual(stop.desc, '')
        self.assertEqual(str(stop.lat), '36.425288')
        self.assertEqual(str(stop.lon), '-117.133162')
        self.assertEqual(stop.zone, None)
        self.assertEqual(stop.url, '')
        self.assertEqual(stop.location_type, '')
        self.assertEqual(stop.parent_station, None)
        self.assertEqual(stop.timezone, '')
        self.assertEqual(stop.wheelchair_boarding, '')
        self.assertEqual({'platform_code': '7'}, stop.extra_data)
        expected = {'extra_columns': {'Stop': ['platform_code']}}
        self.assertEqual(expected, self.feed.meta)

    def test_import_stops_txt_duplicate(self):
        stops_txt = StringIO("""\
stop_id,stop_name,stop_desc,stop_lat,stop_lon
FUR_CREEK_RES,Furnace Creek Resort (Demo),,36.425288,-117.133162
FUR_CREEK_RES,Furnace Creek Resort,,36.42,-117.13
""")
        Stop.import_txt(stops_txt, self.feed)
        stop = Stop.objects.get()
        self.assertEqual(stop.feed, self.feed)
        self.assertEqual(stop.stop_id, 'FUR_CREEK_RES')
        self.assertEqual(stop.name, 'Furnace Creek Resort (Demo)')

    def test_import_stops_txt_maximal(self):
        stops_txt = StringIO("""\
stop_id,stop_code,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url,\
location_type,parent_station,stop_timezone,wheelchair_boarding
FUR_CREEK_STA,,Furnace Creek Station,"Our Station",36.425288,-117.133162,A,\
http://example.com,1,,America/Los_Angeles,1
FUR_CREEK_RES,FC,Furnace Creek Resort,,36.425288,-117.133162,A,\
http://example.com/fcr,0,FUR_CREEK_STA,
FEZ_CREEK_STA,,Fez Creek Station,"Our Station",36.425288,-117.133162,A,\
http://example.com,1,,America/Los_Angeles
FEZ_CREEK_RES,FC,Fez Creek Resort,,36.425288,-117.133162,A,\
http://example.com/fcr,0,FEZ_CREEK_STA,
""")
        Stop.import_txt(stops_txt, self.feed)
        self.assertEqual(Stop.objects.count(), 4)

        station = Stop.objects.get(stop_id='FUR_CREEK_STA')
        zone = Zone.objects.get(feed=self.feed, zone_id='A')
        self.assertEqual(station.code, '')
        self.assertEqual(station.name, 'Furnace Creek Station')
        self.assertEqual(station.desc, 'Our Station')
        self.assertEqual(str(station.lat), '36.425288')
        self.assertEqual(str(station.lon), '-117.133162')
        self.assertEqual(station.zone, zone)
        self.assertEqual(station.url, 'http://example.com')
        self.assertEqual(station.location_type, '1')
        self.assertEqual(station.parent_station, None)
        self.assertEqual(station.timezone, 'America/Los_Angeles')
        self.assertEqual(station.wheelchair_boarding, '1')

        stop = Stop.objects.get(stop_id='FUR_CREEK_RES')
        self.assertEqual(stop.code, 'FC')
        self.assertEqual(stop.name, 'Furnace Creek Resort')
        self.assertEqual(stop.desc, '')
        self.assertEqual(str(stop.lat), '36.425288')
        self.assertEqual(str(stop.lon), '-117.133162')
        self.assertEqual(stop.zone, zone)
        self.assertEqual(stop.url, 'http://example.com/fcr')
        self.assertEqual(stop.location_type, '0')
        self.assertEqual(stop.parent_station, station)
        self.assertEqual(stop.timezone, '')

        stop2 = Stop.objects.get(stop_id='FEZ_CREEK_RES')
        self.assertEqual(stop2.parent_station.stop_id, 'FEZ_CREEK_STA')

    def test_import_stops_txt_stop_before_station(self):
        '''parent_station is set when the stop comes first'''
        stops_txt = StringIO("""\
stop_id,stop_code,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url,\
location_type,parent_station,stop_timezone
FUR_CREEK_RES,FC,Furnace Creek Resort,,36.425288,-117.133162,A,\
http://example.com/fcr,0,FUR_CREEK_STA,
FUR_CREEK_STA,,Furnace Creek Station,"Our Station",36.425288,-117.133162,A,\
http://example.com,1,,America/Los_Angeles
""")
        Stop.import_txt(stops_txt, self.feed)
        self.assertEqual(Stop.objects.count(), 2)
        station = Stop.objects.get(stop_id='FUR_CREEK_STA')
        stop = Stop.objects.get(stop_id='FUR_CREEK_RES')
        self.assertEqual(stop.parent_station, station)

    def test_import_stops_txt_stop_before_station_plus_extra(self):
        stops_txt = StringIO("""\
stop_id,stop_code,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url,\
location_type,parent_station,stop_timezone,extra
FUR_CREEK_RES,FC,Furnace Creek Resort,,36.425288,-117.133162,A,\
http://example.com/fcr,0,FUR_CREEK_STA,,a stop
FUR_CREEK_STA,,Furnace Creek Station,"Our Station",36.425288,-117.133162,A,\
http://example.com,1,,America/Los_Angeles,a station
""")
        Stop.import_txt(stops_txt, self.feed)
        self.assertEqual(Stop.objects.count(), 2)
        station = Stop.objects.get(stop_id='FUR_CREEK_STA')
        stop = Stop.objects.get(stop_id='FUR_CREEK_RES')
        self.assertEqual(stop.parent_station, station)
        self.assertEqual(stop.extra_data, {'extra': 'a stop'})
        self.assertEqual(station.extra_data, {'extra': 'a station'})
        expected = {'extra_columns': {'Stop': ['extra']}}
        self.assertEqual(expected, self.feed.meta)

    def test_import_stops_txt_bom(self):
        stops_txt = StringIO(bom_prefix_csv("""\
stop_id,stop_code,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url,\
location_type,parent_station,stop_timezone
FUR_CREEK_RES,FC,Furnace Creek Resort,,36.425288,-117.133162,A,\
http://example.com/fcr,0,FUR_CREEK_STA,
FUR_CREEK_STA,,Furnace Creek Station,"Our Station",36.425288,-117.133162,A,\
http://example.com,1,,America/Los_Angeles
"""))
        Stop.import_txt(stops_txt, self.feed)
        self.assertEqual(Stop.objects.count(), 2)
        station = Stop.objects.get(stop_id='FUR_CREEK_STA')
        stop = Stop.objects.get(stop_id='FUR_CREEK_RES')
        self.assertEqual(stop.parent_station, station)

    def test_import_stops_txt_parent_is_space(self):
        """If parent station is an empty string, then leave unset

        Issue #36
        """
        stops_txt = StringIO("""\
stop_id,stop_name,stop_desc,stop_lat,stop_lon,parent_station
FUR_CREEK_RES,Furnace Creek Resort (Demo),,36.425288,-117.133162," "
""")
        Stop.import_txt(stops_txt, self.feed)
        stop = Stop.objects.get()
        self.assertEqual(stop.parent_station, None)

    def test_import_stops_txt_strip_spaces(self):
        """A whitespace-only field is treated as empty.

        From SFMTA feed.
        """
        stops_txt = StringIO("""\
stop_id,stop_code,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url
98,198,2ND ST & MARKET ST, ,37.789255,-122.401225, ,%s
""" % ' ')  # Get around flake8 warning on trailing whitespace
        Stop.import_txt(stops_txt, self.feed)
        stop = Stop.objects.get()
        self.assertEqual(stop.name, '2ND ST & MARKET ST')
        self.assertEqual(stop.desc, '')
        self.assertEqual(stop.zone, None)
        self.assertEqual(stop.url, '')

    def test_export_stops_txt_none(self):
        stops_txt = Stop.export_txt(self.feed)
        self.assertFalse(stops_txt)

    def test_export_stops_txt_minimal(self):
        Stop.objects.create(
            feed=self.feed, stop_id='FUR_CREEK_RES',
            name='Furnace Creek Resort (Demo)',
            point="POINT(-117.133162 36.425288)")
        stops_txt = Stop.export_txt(self.feed)
        self.assertEqual(stops_txt, """\
stop_id,stop_name,stop_lat,stop_lon
FUR_CREEK_RES,Furnace Creek Resort (Demo),36.425288,-117.133162
""")

    def test_export_stops_utf8(self):
        Stop.objects.create(
            feed=self.feed, stop_id=6071,
            name=b'The Delta Caf\x82'.decode('latin1'),
            point='POINT(-95.975834 36.114554)')
        stops_txt = force_utf8(Stop.export_txt(self.feed))
        self.assertEqual(stops_txt, b"""\
stop_id,stop_name,stop_lat,stop_lon
6071,The Delta Caf\xc2\x82,36.114554,-95.975834
""")

    def test_export_stops_extra_data(self):
        self.feed.meta = {'extra_columns': {'Stop': ['extra']}}
        self.feed.save()
        Stop.objects.create(
            feed=self.feed, stop_id='FUR_CREEK_RES',
            name='Furnace Creek Resort (Demo)',
            point="POINT(-117.133162 36.425288)",
            extra_data={'extra': '7'})
        stops_txt = Stop.export_txt(self.feed)
        self.assertEqual(stops_txt, """\
stop_id,stop_name,stop_lat,stop_lon,extra
FUR_CREEK_RES,Furnace Creek Resort (Demo),36.425288,-117.133162,7
""")

    def test_update_geometry_on_stop_save(self):
        route = Route.objects.create(feed=self.feed, rtype=3)
        trip = Trip.objects.create(route=route)
        s1 = Stop.objects.create(
            feed=self.feed, point="POINT(-117.133162 36.425288)")
        s2 = Stop.objects.create(
            feed=self.feed, point="POINT(-117.13 36.42)")
        StopTime.objects.create(stop=s1, trip=trip, stop_sequence=1)
        StopTime.objects.create(stop=s2, trip=trip, stop_sequence=2)

        # Starts unset
        trip = Trip.objects.get(id=trip.id)
        route = Route.objects.get(id=route.id)
        self.assertFalse(trip.geometry)

        # Stop save
        s1.save()

        # Now set
        trip = Trip.objects.get(id=trip.id)
        route = Route.objects.get(id=route.id)
        self.assertEqual(
            trip.geometry.coords,
            ((-117.133162, 36.425288), (-117.13, 36.42)))
        self.assertEqual(route.geometry,
                         MultiLineString(trip.geometry, srid=4326))



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from .agency import Agency
from .block import Block
from .fare import Fare
from .fare_rule import FareRule
from .feed import Feed
from .feed_info import FeedInfo
from .frequency import Frequency
from .route import Route
from .service import Service
from .service_date import ServiceDate
from .shape import Shape, ShapePoint
from .stop import Stop
from .stop_time import StopTime
from .transfer import Transfer
from .trip import Trip
from .zone import Zone

# pyflakes be quiet
__models = (
    Agency, Block, Fare, FareRule, Feed, FeedInfo, Frequency, Route, Service,
    ServiceDate, Shape, ShapePoint, Stop, StopTime, Transfer, Trip, Zone)



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import unicode_literals

import os
import shutil
import tempfile
import zipfile

from django.test import TestCase
from django.utils.six import text_type

from multigtfs.models import (
    Agency, Block, Fare, FareRule, Feed, FeedInfo, Frequency,
    Route, Service, ServiceDate, Shape, ShapePoint, Stop, StopTime, Transfer,
    Trip, Zone)

my_dir = os.path.dirname(__file__)
fixtures_dir = os.path.join(my_dir, 'fixtures')


class FeedTest(TestCase):

    def setUp(self):
        self.temp_path = None
        self.temp_dir = None

    def tearDown(self):
        if self.temp_path:
            os.unlink(self.temp_path)
        if self.temp_dir:
            shutil.rmtree(self.temp_dir)

    def normalize(self, feed):
        '''Normalize a feed - line seperators, etc.'''
        feed = feed.replace(b'\r\n', b'\n').strip()
        lines = feed.split(b'\n')
        header = lines.pop(0)
        lines.sort()
        return header + b'\n' + b'\n'.join(lines) + b'\n'

    def test_string(self):
        feed = Feed.objects.create()
        self.assertEqual(str(feed), '%d' % feed.id)
        feed.name = 'Test'
        self.assertEqual(str(feed), '%d Test' % feed.id)

    def test_import_gtfs_test1(self, gtfs_obj=None):
        '''Try importing test1.zip

        test1.zip was downloaded from
        http://timetablepublisher.googlecode.com/files/GTFS%20Test%20Data.zip
        on April 14th, 2012

        dv/trips.txt contains multiple lines with trip id BFC1 and BFC2:

        BFC,W,BFC1,to Furnace Creek Resort,0,1
        BFC,W,BFC2,to Bullfrog,1,2
        BFC,S,BFC1,to Furnace Creek Resort,0,1
        BFC,S,BFC2,to Bullfrog,1,2
        BFC,U,BFC1,to Furnace Creek Resort,0,1
        BFC,U,BFC2,to Bullfrog,1,2

        Before 0.4.0, these were imported with a many-to-many relation to
        services table.  After 0.4.0, the service ID was changed to a single
        foriegn key, so only the 'W' service is recorded.
        '''
        gtfs_obj = gtfs_obj or os.path.abspath(
            os.path.join(fixtures_dir, 'test1.zip'))
        feed = Feed.objects.create()
        feed.import_gtfs(gtfs_obj)
        self.assertEqual(Agency.objects.count(), 1)
        self.assertEqual(Block.objects.count(), 6)
        self.assertEqual(Fare.objects.count(), 0)
        self.assertEqual(FareRule.objects.count(), 0)
        self.assertEqual(Feed.objects.count(), 1)
        self.assertEqual(FeedInfo.objects.count(), 0)
        self.assertEqual(Frequency.objects.count(), 11)
        self.assertEqual(Route.objects.count(), 5)
        self.assertEqual(Service.objects.count(), 3)
        service_W = Service.objects.get(service_id='W')
        service_S = Service.objects.get(service_id='S')
        service_U = Service.objects.get(service_id='U')
        self.assertEqual(service_W.trip_set.count(), 9)
        self.assertEqual(service_S.trip_set.count(), 2)  # 2 trips dropped
        self.assertEqual(service_U.trip_set.count(), 0)  # 2 trips dropped
        self.assertEqual(ServiceDate.objects.count(), 1)
        self.assertEqual(Shape.objects.count(), 0)
        self.assertEqual(Stop.objects.count(), 9)
        self.assertEqual(StopTime.objects.count(), 28)
        self.assertEqual(Transfer.objects.count(), 0)
        self.assertEqual(Trip.objects.count(), 11)
        self.assertEqual(Zone.objects.count(), 0)

    def test_import_gtfs_test1_extracted(self):
        '''Import test1.zip as an extracted folder'''
        test_path = os.path.abspath(os.path.join(fixtures_dir, 'test1.zip'))
        self.temp_dir = tempfile.mkdtemp()
        z = zipfile.ZipFile(test_path)
        z.extractall(self.temp_dir)
        self.test_import_gtfs_test1(self.temp_dir)

    def test_import_gtfs_test2(self):
        '''Try importing test2.zip

        test2.zip was downloaded from
        https://developers.google.com/transit/gtfs/examples/sample-feed.zip
        on April 14th, 2012
        '''
        test_path = os.path.abspath(os.path.join(fixtures_dir, 'test2.zip'))
        feed = Feed.objects.create()
        feed.import_gtfs(test_path)

        self.assertEqual(Agency.objects.count(), 1)
        self.assertEqual(Block.objects.count(), 2)
        self.assertEqual(Fare.objects.count(), 2)
        self.assertEqual(FareRule.objects.count(), 4)
        self.assertEqual(Feed.objects.count(), 1)
        self.assertEqual(FeedInfo.objects.count(), 0)
        self.assertEqual(Frequency.objects.count(), 11)
        self.assertEqual(Route.objects.count(), 5)
        self.assertEqual(Service.objects.count(), 2)
        self.assertEqual(ServiceDate.objects.count(), 1)
        self.assertEqual(Shape.objects.count(), 0)
        self.assertEqual(Stop.objects.count(), 9)
        self.assertEqual(StopTime.objects.count(), 28)
        self.assertEqual(Transfer.objects.count(), 0)
        self.assertEqual(Trip.objects.count(), 11)
        self.assertEqual(Zone.objects.count(), 0)

    def test_import_gtfs_test3(self):
        '''Try importing test3.zip

        test3.zip is a version of the GTFS feed from
        http://www.tulsatransit.org/gtfs/google_transit.zip
        as it was around February 2013.  It has been reduced to one route,
        trip, and shape, along with associated stops and other records.
        '''
        test_path = os.path.abspath(os.path.join(fixtures_dir, 'test3.zip'))
        feed = Feed.objects.create()
        feed.import_gtfs(test_path)
        self.assertEqual(Agency.objects.count(), 1)
        self.assertEqual(Block.objects.count(), 0)
        self.assertEqual(Fare.objects.count(), 1)
        self.assertEqual(FareRule.objects.count(), 1)
        self.assertEqual(Feed.objects.count(), 1)
        self.assertEqual(FeedInfo.objects.count(), 1)
        self.assertEqual(Frequency.objects.count(), 0)
        self.assertEqual(Route.objects.count(), 1)
        self.assertEqual(Service.objects.count(), 1)
        self.assertEqual(ServiceDate.objects.count(), 7)
        self.assertEqual(Shape.objects.count(), 1)
        self.assertEqual(ShapePoint.objects.count(), 61)
        self.assertEqual(Stop.objects.count(), 24)
        self.assertEqual(StopTime.objects.count(), 25)
        self.assertEqual(Transfer.objects.count(), 0)
        self.assertEqual(Trip.objects.count(), 1)
        self.assertEqual(Zone.objects.count(), 0)

    def test_import_gtfs_test4(self):
        '''Try importing test4.zip

        test4.zip is a version of the GTFS feed from TriMet in Portland
        from February 2015.  It omits calendar.txt in favor of specifying
        all dates in calendar_dates.txt, and includes additional files not
        in the GTFS spec.
        '''
        test_path = os.path.abspath(os.path.join(fixtures_dir, 'test4.zip'))
        feed = Feed.objects.create()
        feed.import_gtfs(test_path)
        self.assertEqual(Agency.objects.count(), 1)
        self.assertEqual(Block.objects.count(), 1)
        self.assertEqual(Fare.objects.count(), 1)
        self.assertEqual(FareRule.objects.count(), 1)
        self.assertEqual(Feed.objects.count(), 1)
        self.assertEqual(FeedInfo.objects.count(), 1)
        self.assertEqual(Frequency.objects.count(), 0)
        self.assertEqual(Route.objects.count(), 1)
        self.assertEqual(Service.objects.count(), 1)
        self.assertEqual(ServiceDate.objects.count(), 15)
        self.assertEqual(Shape.objects.count(), 1)
        self.assertEqual(ShapePoint.objects.count(), 131)
        self.assertEqual(Stop.objects.count(), 10)
        self.assertEqual(StopTime.objects.count(), 10)
        self.assertEqual(Transfer.objects.count(), 2)
        self.assertEqual(Trip.objects.count(), 1)
        self.assertEqual(Zone.objects.count(), 1)

    def test_export_gtfs_test1(self):
        '''Try exporting test1.zip'''
        test_path = os.path.abspath(os.path.join(fixtures_dir, 'test1.zip'))
        feed = Feed.objects.create()
        feed.import_gtfs(test_path)
        file_id, self.temp_path = tempfile.mkstemp()
        os.close(file_id)
        feed.export_gtfs(self.temp_path)
        z_in = zipfile.ZipFile(test_path, 'r')
        z_out = zipfile.ZipFile(self.temp_path, 'r')
        self.assertEqual(
            z_in.namelist(),
            ['dv/', 'dv/.DS_Store', '__MACOSX/', '__MACOSX/dv/',
             '__MACOSX/dv/._.DS_Store',
             'dv/agency.txt',
             'dv/calendar.txt',
             'dv/calendar_dates.txt',
             'dv/frequencies.txt',
             'dv/routes.txt',
             'dv/stop_times.txt',
             'dv/stops.txt',
             'dv/trips.txt'])
        self.assertEqual(
            z_out.namelist(),
            ['agency.txt',
             'calendar.txt',
             'calendar_dates.txt',
             'frequencies.txt',
             'routes.txt',
             'stop_times.txt',
             'stops.txt',
             'trips.txt'])

        agency_in = self.normalize(z_in.read('dv/agency.txt'))
        agency_out = self.normalize(z_out.read('agency.txt'))
        self.assertEqual(agency_in, agency_out)

        calendar_in = self.normalize(z_in.read('dv/calendar.txt'))
        calendar_out = self.normalize(z_out.read('calendar.txt'))
        self.assertEqual(calendar_in, calendar_out)

        cdates_in = self.normalize(z_in.read('dv/calendar_dates.txt'))
        cdates_out = self.normalize(z_out.read('calendar_dates.txt'))
        self.assertEqual(cdates_in, cdates_out)

        self.assertFalse('dv/fare_attributes.txt' in z_in.namelist())
        self.assertFalse('feed/fare_attributes.txt' in z_out.namelist())

        self.assertFalse('dv/fare_rules.txt' in z_in.namelist())
        self.assertFalse('feed/fare_rules.txt' in z_out.namelist())

        self.assertFalse('dv/feed_info.txt' in z_in.namelist())
        self.assertFalse('feed/feed_info.txt' in z_out.namelist())

        freq_in = self.normalize(z_in.read('dv/frequencies.txt'))
        self.assertEqual(freq_in, b'''\
trip_id,start_time,end_time,headway_secs
CITY1,10:00:00,15:59:59,1800
CITY1,16:00:00,18:59:59,600
CITY1,19:00:00,22:00:00,1800
CITY1,6:00:00,7:59:59,1800
CITY1,8:00:00,9:59:59,600
CITY2,10:00:00,15:59:59,1800
CITY2,16:00:00,18:59:59,600
CITY2,19:00:00,22:00:00,1800
CITY2,6:00:00,7:59:59,1800
CITY2,8:00:00,9:59:59,600
STBA,6:00:00,22:00:00,1800
''')
        freq_out = self.normalize(z_out.read('frequencies.txt'))
        self.assertNotEqual(freq_out, "Le Freak, C'est Chic")
        self.assertEqual(freq_out, b'''\
trip_id,start_time,end_time,headway_secs
CITY1,06:00:00,07:59:59,1800
CITY1,08:00:00,09:59:59,600
CITY1,10:00:00,15:59:59,1800
CITY1,16:00:00,18:59:59,600
CITY1,19:00:00,22:00:00,1800
CITY2,06:00:00,07:59:59,1800
CITY2,08:00:00,09:59:59,600
CITY2,10:00:00,15:59:59,1800
CITY2,16:00:00,18:59:59,600
CITY2,19:00:00,22:00:00,1800
STBA,06:00:00,22:00:00,1800
''')

        routes_in = self.normalize(z_in.read('dv/routes.txt'))
        routes_out = self.normalize(z_out.read('routes.txt'))
        self.assertEqual(routes_in, routes_out)

        self.assertFalse('dv/shapes.txt' in z_in.namelist())
        self.assertFalse('feed/shapes.txt' in z_out.namelist())

        stimes_in = self.normalize(z_in.read('dv/stop_times.txt'))
        self.assertEqual(stimes_in, b"""\
trip_id,arrival_time,departure_time,stop_id,stop_sequence
AAMV1,8:00:00,8:00:00,BEATTY_AIRPORT,1
AAMV1,9:00:00,9:00:00,AMV,2
AAMV2,10:00:00,10:00:00,AMV,1
AAMV2,11:00:00,11:00:00,BEATTY_AIRPORT,2
AAMV3,13:00:00,13:00:00,BEATTY_AIRPORT,1
AAMV3,14:00:00,14:00:00,AMV,2
AAMV4,15:00:00,15:00:00,AMV,1
AAMV4,16:00:00,16:00:00,BEATTY_AIRPORT,2
AB1,8:00:00,8:00:00,BEATTY_AIRPORT,1
AB1,8:10:00,8:15:00,BULLFROG,2
AB2,12:05:00,12:05:00,BULLFROG,1
AB2,12:15:00,12:15:00,BEATTY_AIRPORT,2
BFC1,8:20:00,8:20:00,BULLFROG,1
BFC1,9:20:00,9:20:00,FUR_CREEK_RES,2
BFC2,11:00:00,11:00:00,FUR_CREEK_RES,1
BFC2,12:00:00,12:00:00,BULLFROG,2
CITY1,6:00:00,6:00:00,STAGECOACH,1
CITY1,6:05:00,6:07:00,NANAA,2
CITY1,6:12:00,6:14:00,NADAV,3
CITY1,6:19:00,6:21:00,DADAN,4
CITY1,6:26:00,6:28:00,EMSI,5
CITY2,6:28:00,6:30:00,EMSI,1
CITY2,6:35:00,6:37:00,DADAN,2
CITY2,6:42:00,6:44:00,NADAV,3
CITY2,6:49:00,6:51:00,NANAA,4
CITY2,6:56:00,6:58:00,STAGECOACH,5
STBA,6:00:00,6:00:00,STAGECOACH,1
STBA,6:20:00,6:20:00,BEATTY_AIRPORT,2
""")
        stimes_out = self.normalize(z_out.read('stop_times.txt'))
        self.assertEqual(stimes_out, b"""\
trip_id,arrival_time,departure_time,stop_id,stop_sequence
AAMV1,08:00:00,08:00:00,BEATTY_AIRPORT,1
AAMV1,09:00:00,09:00:00,AMV,2
AAMV2,10:00:00,10:00:00,AMV,1
AAMV2,11:00:00,11:00:00,BEATTY_AIRPORT,2
AAMV3,13:00:00,13:00:00,BEATTY_AIRPORT,1
AAMV3,14:00:00,14:00:00,AMV,2
AAMV4,15:00:00,15:00:00,AMV,1
AAMV4,16:00:00,16:00:00,BEATTY_AIRPORT,2
AB1,08:00:00,08:00:00,BEATTY_AIRPORT,1
AB1,08:10:00,08:15:00,BULLFROG,2
AB2,12:05:00,12:05:00,BULLFROG,1
AB2,12:15:00,12:15:00,BEATTY_AIRPORT,2
BFC1,08:20:00,08:20:00,BULLFROG,1
BFC1,09:20:00,09:20:00,FUR_CREEK_RES,2
BFC2,11:00:00,11:00:00,FUR_CREEK_RES,1
BFC2,12:00:00,12:00:00,BULLFROG,2
CITY1,06:00:00,06:00:00,STAGECOACH,1
CITY1,06:05:00,06:07:00,NANAA,2
CITY1,06:12:00,06:14:00,NADAV,3
CITY1,06:19:00,06:21:00,DADAN,4
CITY1,06:26:00,06:28:00,EMSI,5
CITY2,06:28:00,06:30:00,EMSI,1
CITY2,06:35:00,06:37:00,DADAN,2
CITY2,06:42:00,06:44:00,NADAV,3
CITY2,06:49:00,06:51:00,NANAA,4
CITY2,06:56:00,06:58:00,STAGECOACH,5
STBA,06:00:00,06:00:00,STAGECOACH,1
STBA,06:20:00,06:20:00,BEATTY_AIRPORT,2
""")

        stops_in = self.normalize(z_in.read('dv/stops.txt'))
        self.assertEqual(stops_in, b"""\
stop_id,stop_name,stop_desc,stop_lat,stop_lon
AMV,Amargosa Valley (Demo),,36.641496,-116.40094
BEATTY_AIRPORT,Nye County Airport (Demo),,36.868446,-116.784582
BULLFROG,Bullfrog (Demo),,36.88108,-116.81797
DADAN,Doing Ave / D Ave N (Demo),,36.909489,-116.768242
EMSI,E Main St / S Irving St (Demo),,36.905697,-116.76218
FUR_CREEK_RES,Furnace Creek Resort (Demo),,36.425288,-117.133162
NADAV,North Ave / D Ave N (Demo),,36.914893,-116.76821
NANAA,North Ave / N A Ave (Demo),,36.914944,-116.761472
STAGECOACH,Stagecoach Hotel & Casino (Demo),,36.915682,-116.751677
""")
        stops_out = self.normalize(z_out.read('stops.txt'))
        self.assertEqual(stops_out, b"""\
stop_id,stop_name,stop_lat,stop_lon
AMV,Amargosa Valley (Demo),36.641496,-116.40094
BEATTY_AIRPORT,Nye County Airport (Demo),36.868446,-116.784582
BULLFROG,Bullfrog (Demo),36.88108,-116.81797
DADAN,Doing Ave / D Ave N (Demo),36.909489,-116.768242
EMSI,E Main St / S Irving St (Demo),36.905697,-116.76218
FUR_CREEK_RES,Furnace Creek Resort (Demo),36.425288,-117.133162
NADAV,North Ave / D Ave N (Demo),36.914893,-116.76821
NANAA,North Ave / N A Ave (Demo),36.914944,-116.761472
STAGECOACH,Stagecoach Hotel & Casino (Demo),36.915682,-116.751677
""")

        self.assertFalse('dv/transfers.txt' in z_in.namelist())
        self.assertFalse('feed/transfers.txt' in z_out.namelist())

        # Duplicate trips are dropped
        trips_out = self.normalize(z_out.read('trips.txt'))
        self.assertEqual(trips_out, b"""\
route_id,service_id,trip_id,trip_headsign,direction_id,block_id
AAMV,S,AAMV3,to Amargosa Valley,0,4
AAMV,S,AAMV4,to Airport,1,4
AAMV,W,AAMV1,to Amargosa Valley,0,3
AAMV,W,AAMV2,to Airport,1,3
AB,W,AB1,to Bullfrog,0,1
AB,W,AB2,to Airport,1,2
BFC,W,BFC1,to Furnace Creek Resort,0,1
BFC,W,BFC2,to Bullfrog,1,2
CITY,W,CITY1,to Somewhere,0,44
CITY,W,CITY2,to Nowhere,1,45
STBA,W,STBA,Shuttle,1,2
""")

    def test_export_gtfs_test2(self):
        '''Try exporting test2.zip'''
        test_path = os.path.abspath(os.path.join(fixtures_dir, 'test2.zip'))
        feed = Feed.objects.create()
        feed.import_gtfs(test_path)
        file_id, self.temp_path = tempfile.mkstemp()
        os.close(file_id)
        feed.export_gtfs(self.temp_path)
        z_in = zipfile.ZipFile(test_path, 'r')
        z_out = zipfile.ZipFile(self.temp_path, 'r')

        self.assertEqual(
            z_in.namelist(),
            ['agency.txt',
             'calendar.txt',
             'calendar_dates.txt',
             'fare_attributes.txt',
             'fare_rules.txt',
             'frequencies.txt',
             'routes.txt',
             'shapes.txt',
             'stop_times.txt',
             'stops.txt',
             'trips.txt'])
        self.assertEqual(
            z_out.namelist(),
            ['agency.txt',
             'calendar.txt',
             'calendar_dates.txt',
             'fare_attributes.txt',
             'fare_rules.txt',
             'frequencies.txt',
             'routes.txt',
             'stop_times.txt',
             'stops.txt',
             'trips.txt'])

        agency_in = self.normalize(z_in.read('agency.txt'))
        agency_out = self.normalize(z_out.read('agency.txt'))
        self.assertEqual(agency_in, agency_out)

        calendar_in = self.normalize(z_in.read('calendar.txt'))
        calendar_out = self.normalize(z_out.read('calendar.txt'))
        self.assertEqual(calendar_in, calendar_out)

        cdates_in = self.normalize(z_in.read('calendar_dates.txt'))
        cdates_out = self.normalize(z_out.read('calendar_dates.txt'))
        self.assertEqual(cdates_in, cdates_out)

        # source fare_attributes.txt has unneeded transfer_duration column
        fare_in = self.normalize(z_in.read('fare_attributes.txt'))
        self.assertEqual(fare_in, b'''\
fare_id,price,currency_type,payment_method,transfers,transfer_duration
a,5.25,USD,0,0,
p,1.25,USD,0,0,
''')
        # Sometimes 5.2500, sometimes 5.25
        fare_a = Fare.objects.get(fare_id='a')
        fare_p = Fare.objects.get(fare_id='p')
        s_fare_a = text_type(fare_a.price).encode('utf-8')
        s_fare_p = text_type(fare_p.price).encode('utf-8')
        fare_out = z_out.read('fare_attributes.txt')
        self.assertEqual(fare_out, b'''\
fare_id,price,currency_type,payment_method,transfers
a,''' + s_fare_a + b''',USD,0,0
p,''' + s_fare_p + b''',USD,0,0
''')

        # source fare_rules.txt has unneeded columns
        fare_rules_in = self.normalize(z_in.read('fare_rules.txt'))
        self.assertEqual(fare_rules_in, b'''\
fare_id,route_id,origin_id,destination_id,contains_id
a,AAMV,,,
p,AB,,,
p,BFC,,,
p,STBA,,,
''')
        fare_rules_out = self.normalize(z_out.read('fare_rules.txt'))
        self.assertEqual(fare_rules_out, b'''\
fare_id,route_id
a,AAMV
p,AB
p,BFC
p,STBA
''')

        self.assertFalse('feed_info.txt' in z_in.namelist())
        self.assertFalse('feed/feed_info.txt' in z_out.namelist())

        freq_in = self.normalize(z_in.read('frequencies.txt'))
        self.assertEqual(freq_in, b'''\
trip_id,start_time,end_time,headway_secs
CITY1,10:00:00,15:59:59,1800
CITY1,16:00:00,18:59:59,600
CITY1,19:00:00,22:00:00,1800
CITY1,6:00:00,7:59:59,1800
CITY1,8:00:00,9:59:59,600
CITY2,10:00:00,15:59:59,1800
CITY2,16:00:00,18:59:59,600
CITY2,19:00:00,22:00:00,1800
CITY2,6:00:00,7:59:59,1800
CITY2,8:00:00,9:59:59,600
STBA,6:00:00,22:00:00,1800
''')
        freq_out = self.normalize(z_out.read('frequencies.txt'))
        self.assertEqual(freq_out, b'''\
trip_id,start_time,end_time,headway_secs
CITY1,06:00:00,07:59:59,1800
CITY1,08:00:00,09:59:59,600
CITY1,10:00:00,15:59:59,1800
CITY1,16:00:00,18:59:59,600
CITY1,19:00:00,22:00:00,1800
CITY2,06:00:00,07:59:59,1800
CITY2,08:00:00,09:59:59,600
CITY2,10:00:00,15:59:59,1800
CITY2,16:00:00,18:59:59,600
CITY2,19:00:00,22:00:00,1800
STBA,06:00:00,22:00:00,1800
''')

        routes_in = self.normalize(z_in.read('routes.txt'))
        self.assertEqual(routes_in, b"""\
route_id,agency_id,route_short_name,route_long_name,route_desc,route_type\
,route_url,route_color,route_text_color
AAMV,DTA,50,Airport - Amargosa Valley,,3,,,
AB,DTA,10,Airport - Bullfrog,,3,,,
BFC,DTA,20,Bullfrog - Furnace Creek Resort,,3,,,
CITY,DTA,40,City,,3,,,
STBA,DTA,30,Stagecoach - Airport Shuttle,,3,,,
""")
        routes_out = self.normalize(z_out.read('routes.txt'))
        self.assertEqual(routes_out, b"""\
route_id,agency_id,route_short_name,route_long_name,route_type
AAMV,DTA,50,Airport - Amargosa Valley,3
AB,DTA,10,Airport - Bullfrog,3
BFC,DTA,20,Bullfrog - Furnace Creek Resort,3
CITY,DTA,40,City,3
STBA,DTA,30,Stagecoach - Airport Shuttle,3
""")

        shapes_out = self.normalize(z_in.read('shapes.txt'))
        self.assertEqual(shapes_out, b'''\
shape_id,shape_pt_lat,shape_pt_lon,shape_pt_sequence,shape_dist_traveled

''')
        self.assertFalse('feed/shapes.txt' in z_out.namelist())

        stimes_in = self.normalize(z_in.read('stop_times.txt'))
        self.assertEqual(stimes_in, b"""\
trip_id,arrival_time,departure_time,stop_id,stop_sequence,stop_headsign,\
pickup_type,drop_off_time,shape_dist_traveled
AAMV1,8:00:00,8:00:00,BEATTY_AIRPORT,1
AAMV1,9:00:00,9:00:00,AMV,2
AAMV2,10:00:00,10:00:00,AMV,1
AAMV2,11:00:00,11:00:00,BEATTY_AIRPORT,2
AAMV3,13:00:00,13:00:00,BEATTY_AIRPORT,1
AAMV3,14:00:00,14:00:00,AMV,2
AAMV4,15:00:00,15:00:00,AMV,1
AAMV4,16:00:00,16:00:00,BEATTY_AIRPORT,2
AB1,8:00:00,8:00:00,BEATTY_AIRPORT,1,,,,
AB1,8:10:00,8:15:00,BULLFROG,2,,,,
AB2,12:05:00,12:05:00,BULLFROG,1,,,,
AB2,12:15:00,12:15:00,BEATTY_AIRPORT,2
BFC1,8:20:00,8:20:00,BULLFROG,1
BFC1,9:20:00,9:20:00,FUR_CREEK_RES,2
BFC2,11:00:00,11:00:00,FUR_CREEK_RES,1
BFC2,12:00:00,12:00:00,BULLFROG,2
CITY1,6:00:00,6:00:00,STAGECOACH,1,,,,
CITY1,6:05:00,6:07:00,NANAA,2,,,,
CITY1,6:12:00,6:14:00,NADAV,3,,,,
CITY1,6:19:00,6:21:00,DADAN,4,,,,
CITY1,6:26:00,6:28:00,EMSI,5,,,,
CITY2,6:28:00,6:30:00,EMSI,1,,,,
CITY2,6:35:00,6:37:00,DADAN,2,,,,
CITY2,6:42:00,6:44:00,NADAV,3,,,,
CITY2,6:49:00,6:51:00,NANAA,4,,,,
CITY2,6:56:00,6:58:00,STAGECOACH,5,,,,
STBA,6:00:00,6:00:00,STAGECOACH,1,,,,
STBA,6:20:00,6:20:00,BEATTY_AIRPORT,2,,,,
""")
        stimes_out = self.normalize(z_out.read('stop_times.txt'))
        self.assertEqual(stimes_out, b"""\
trip_id,arrival_time,departure_time,stop_id,stop_sequence
AAMV1,08:00:00,08:00:00,BEATTY_AIRPORT,1
AAMV1,09:00:00,09:00:00,AMV,2
AAMV2,10:00:00,10:00:00,AMV,1
AAMV2,11:00:00,11:00:00,BEATTY_AIRPORT,2
AAMV3,13:00:00,13:00:00,BEATTY_AIRPORT,1
AAMV3,14:00:00,14:00:00,AMV,2
AAMV4,15:00:00,15:00:00,AMV,1
AAMV4,16:00:00,16:00:00,BEATTY_AIRPORT,2
AB1,08:00:00,08:00:00,BEATTY_AIRPORT,1
AB1,08:10:00,08:15:00,BULLFROG,2
AB2,12:05:00,12:05:00,BULLFROG,1
AB2,12:15:00,12:15:00,BEATTY_AIRPORT,2
BFC1,08:20:00,08:20:00,BULLFROG,1
BFC1,09:20:00,09:20:00,FUR_CREEK_RES,2
BFC2,11:00:00,11:00:00,FUR_CREEK_RES,1
BFC2,12:00:00,12:00:00,BULLFROG,2
CITY1,06:00:00,06:00:00,STAGECOACH,1
CITY1,06:05:00,06:07:00,NANAA,2
CITY1,06:12:00,06:14:00,NADAV,3
CITY1,06:19:00,06:21:00,DADAN,4
CITY1,06:26:00,06:28:00,EMSI,5
CITY2,06:28:00,06:30:00,EMSI,1
CITY2,06:35:00,06:37:00,DADAN,2
CITY2,06:42:00,06:44:00,NADAV,3
CITY2,06:49:00,06:51:00,NANAA,4
CITY2,06:56:00,06:58:00,STAGECOACH,5
STBA,06:00:00,06:00:00,STAGECOACH,1
STBA,06:20:00,06:20:00,BEATTY_AIRPORT,2
""")

        stops_in = self.normalize(z_in.read('stops.txt'))
        self.assertEqual(stops_in, b"""\
stop_id,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url
AMV,Amargosa Valley (Demo),,36.641496,-116.40094,,
BEATTY_AIRPORT,Nye County Airport (Demo),,36.868446,-116.784582,,
BULLFROG,Bullfrog (Demo),,36.88108,-116.81797,,
DADAN,Doing Ave / D Ave N (Demo),,36.909489,-116.768242,,
EMSI,E Main St / S Irving St (Demo),,36.905697,-116.76218,,
FUR_CREEK_RES,Furnace Creek Resort (Demo),,36.425288,-117.133162,,
NADAV,North Ave / D Ave N (Demo),,36.914893,-116.76821,,
NANAA,North Ave / N A Ave (Demo),,36.914944,-116.761472,,
STAGECOACH,Stagecoach Hotel & Casino (Demo),,36.915682,-116.751677,,
""")
        stops_out = self.normalize(z_out.read('stops.txt'))
        self.assertEqual(stops_out, b"""\
stop_id,stop_name,stop_lat,stop_lon
AMV,Amargosa Valley (Demo),36.641496,-116.40094
BEATTY_AIRPORT,Nye County Airport (Demo),36.868446,-116.784582
BULLFROG,Bullfrog (Demo),36.88108,-116.81797
DADAN,Doing Ave / D Ave N (Demo),36.909489,-116.768242
EMSI,E Main St / S Irving St (Demo),36.905697,-116.76218
FUR_CREEK_RES,Furnace Creek Resort (Demo),36.425288,-117.133162
NADAV,North Ave / D Ave N (Demo),36.914893,-116.76821
NANAA,North Ave / N A Ave (Demo),36.914944,-116.761472
STAGECOACH,Stagecoach Hotel & Casino (Demo),36.915682,-116.751677
""")

        self.assertFalse('transfers.txt' in z_in.namelist())
        self.assertFalse('feed/transfers.txt' in z_out.namelist())

        trips_in = self.normalize(z_in.read('trips.txt'))
        self.assertEqual(trips_in, b"""\
route_id,service_id,trip_id,trip_headsign,direction_id,block_id,shape_id
AAMV,WE,AAMV1,to Amargosa Valley,0,,
AAMV,WE,AAMV2,to Airport,1,,
AAMV,WE,AAMV3,to Amargosa Valley,0,,
AAMV,WE,AAMV4,to Airport,1,,
AB,FULLW,AB1,to Bullfrog,0,1,
AB,FULLW,AB2,to Airport,1,2,
BFC,FULLW,BFC1,to Furnace Creek Resort,0,1,
BFC,FULLW,BFC2,to Bullfrog,1,2,
CITY,FULLW,CITY1,,0,,
CITY,FULLW,CITY2,,1,,
STBA,FULLW,STBA,Shuttle,,,
""")
        trips_out = self.normalize(z_out.read('trips.txt'))
        self.assertEqual(trips_out, b"""\
route_id,service_id,trip_id,trip_headsign,direction_id,block_id
AAMV,WE,AAMV1,to Amargosa Valley,0,
AAMV,WE,AAMV2,to Airport,1,
AAMV,WE,AAMV3,to Amargosa Valley,0,
AAMV,WE,AAMV4,to Airport,1,
AB,FULLW,AB1,to Bullfrog,0,1
AB,FULLW,AB2,to Airport,1,2
BFC,FULLW,BFC1,to Furnace Creek Resort,0,1
BFC,FULLW,BFC2,to Bullfrog,1,2
CITY,FULLW,CITY1,,0,
CITY,FULLW,CITY2,,1,
STBA,FULLW,STBA,Shuttle,,
""")

    def test_export_gtfs_test3(self):
        '''Try exporting test3.zip'''
        test_path = os.path.abspath(os.path.join(fixtures_dir, 'test3.zip'))
        feed = Feed.objects.create()
        feed.import_gtfs(test_path)
        file_id, self.temp_path = tempfile.mkstemp()
        os.close(file_id)
        feed.export_gtfs(self.temp_path)
        z_in = zipfile.ZipFile(test_path, 'r')
        z_out = zipfile.ZipFile(self.temp_path, 'r')

        self.assertEqual(
            z_in.namelist(),
            ['agency.txt',
             'calendar.txt',
             'calendar_dates.txt',
             'fare_attributes.txt',
             'fare_rules.txt',
             'feed_info.txt',
             'routes.txt',
             'shapes.txt',
             'stop_times.txt',
             'stops.txt',
             'trips.txt'])
        self.assertEqual(
            z_out.namelist(),
            ['agency.txt',
             'calendar.txt',
             'calendar_dates.txt',
             'fare_attributes.txt',
             'fare_rules.txt',
             'feed_info.txt',
             'routes.txt',
             'shapes.txt',
             'stop_times.txt',
             'stops.txt',
             'trips.txt'])

        agency_in = self.normalize(z_in.read('agency.txt'))
        agency_out = self.normalize(z_out.read('agency.txt'))
        self.assertEqual(agency_in, agency_out)

        calendar_in = self.normalize(z_in.read('calendar.txt'))
        calendar_out = self.normalize(z_out.read('calendar.txt'))
        self.assertEqual(calendar_in, calendar_out)

        cdates_in = self.normalize(z_in.read('calendar_dates.txt'))
        cdates_out = self.normalize(z_out.read('calendar_dates.txt'))
        self.assertEqual(cdates_in, cdates_out)

        fare_in = self.normalize(z_in.read('fare_attributes.txt'))
        self.assertEqual(fare_in, b'''\
fare_id,price,currency_type,payment_method,transfers,transfer_duration
adult,1.5000,USD,0,,7200
''')
        # Sometimes 1.5000, sometimes 1.5
        fare_a = Fare.objects.get(fare_id='adult')
        s_fare_a = text_type(fare_a.price).encode('utf-8')
        fare_out = z_out.read('fare_attributes.txt')
        self.assertEqual(fare_out, b'''\
fare_id,price,currency_type,payment_method,transfers,transfer_duration
adult,''' + s_fare_a + b''',USD,0,,7200
''')

        fare_rules_in = self.normalize(z_in.read('fare_rules.txt'))
        fare_rules_out = self.normalize(z_out.read('fare_rules.txt'))
        self.assertEqual(fare_rules_in, fare_rules_out)

        feed_info_in = self.normalize(z_in.read('feed_info.txt'))
        feed_info_out = self.normalize(z_out.read('feed_info.txt'))
        self.assertEqual(feed_info_in, feed_info_out)

        self.assertFalse('frequencies.txt' in z_in.namelist())
        self.assertFalse('frequencies.txt' in z_out.namelist())

        routes_in = self.normalize(z_in.read('routes.txt'))
        routes_out = self.normalize(z_out.read('routes.txt'))
        self.assertEqual(routes_in, routes_out)

        shapes_in = self.normalize(z_in.read('shapes.txt'))
        shapes_out = self.normalize(z_out.read('shapes.txt'))
        self.assertEqual(shapes_in, shapes_out)

        stop_times_in = self.normalize(z_in.read('stop_times.txt'))
        stop_times_out = self.normalize(z_out.read('stop_times.txt'))
        self.assertEqual(stop_times_in, stop_times_out)

        stops_in = self.normalize(z_in.read('stops.txt'))
        stops_out = self.normalize(z_out.read('stops.txt'))
        self.assertEqual(stops_in, stops_out)

        self.assertFalse('transfers.txt' in z_in.namelist())
        self.assertFalse('feed/transfers.txt' in z_out.namelist())

        trips_in = self.normalize(z_in.read('trips.txt'))
        trips_out = self.normalize(z_out.read('trips.txt'))
        self.assertEqual(trips_in, trips_out)

    def test_export_gtfs_test4(self):
        '''Try exporting test4.zip'''
        test_path = os.path.abspath(os.path.join(fixtures_dir, 'test4.zip'))
        feed = Feed.objects.create()
        feed.import_gtfs(test_path)
        file_id, self.temp_path = tempfile.mkstemp()
        os.close(file_id)
        feed.export_gtfs(self.temp_path)
        z_in = zipfile.ZipFile(test_path, 'r')
        z_out = zipfile.ZipFile(self.temp_path, 'r')

        self.assertEqual(
            z_in.namelist(),
            ['agency.txt',
             'calendar_dates.txt',
             'fare_attributes.txt',
             'fare_rules.txt',
             'feed_info.txt',
             'realtime_feeds.txt',
             'route_directions.txt',
             'routes.txt',
             'shapes.txt',
             'stop_features.txt',
             'stop_times.txt',
             'stops.txt',
             'transfers.txt',
             'trips.txt'])
        self.assertEqual(
            z_out.namelist(),
            ['agency.txt',
             'calendar_dates.txt',
             'fare_attributes.txt',
             'fare_rules.txt',
             'feed_info.txt',
             'routes.txt',
             'shapes.txt',
             'stop_times.txt',
             'stops.txt',
             'transfers.txt',
             'trips.txt'])

        agency_in = self.normalize(z_in.read('agency.txt'))
        agency_out = self.normalize(z_out.read('agency.txt'))
        self.assertEqual(agency_in, agency_out)

        cdates_in = self.normalize(z_in.read('calendar_dates.txt'))
        cdates_out = self.normalize(z_out.read('calendar_dates.txt'))
        self.assertEqual(cdates_in, cdates_out)

        fare_in = self.normalize(z_in.read('fare_attributes.txt'))
        self.assertEqual(fare_in, b'''\
fare_id,price,currency_type,payment_method,transfers,transfer_duration
B,2.5,USD,0,,7200
''')
        # Sometimes 2.5000, sometimes 2.5
        fare_b = Fare.objects.get(fare_id='B')
        s_fare_b = text_type(fare_b.price).encode('utf-8')
        fare_out = z_out.read('fare_attributes.txt')
        self.assertEqual(fare_out, b'''\
fare_id,price,currency_type,payment_method,transfers,transfer_duration
B,''' + s_fare_b + b''',USD,0,,7200
''')

        fare_rules_in = self.normalize(z_in.read('fare_rules.txt'))
        fare_rules_out = self.normalize(z_out.read('fare_rules.txt'))
        self.assertEqual(fare_rules_in, b'''\
fare_id,origin_id,route_id,contains_id
B,B,,B
''')
        self.assertEqual(fare_rules_out, b'''\
fare_id,origin_id,contains_id
B,B,B
''')

        routes_in = self.normalize(z_in.read('routes.txt'))
        routes_out = self.normalize(z_out.read('routes.txt'))
        self.assertEqual(routes_in, routes_out)

        shapes_in = self.normalize(z_in.read('shapes.txt'))
        shapes_out = self.normalize(z_out.read('shapes.txt'))
        self.assertEqual(shapes_in, shapes_out)

        stop_times_in = self.normalize(z_in.read('stop_times.txt'))
        stop_times_out = self.normalize(z_out.read('stop_times.txt'))
        self.assertEqual(stop_times_in, stop_times_out)

        stops_in = self.normalize(z_in.read('stops.txt'))
        stops_in_header = stops_in.split(b'\n')[0]
        stops_out = self.normalize(z_out.read('stops.txt'))
        stops_out_header = stops_out.split(b'\n')[0]
        # parent_station in input, not in output
        self.assertEqual(stops_in_header, b'''\
stop_id,stop_code,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url,\
location_type,parent_station,direction,position''')
        self.assertEqual(stops_out_header, b'''\
stop_id,stop_code,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url,\
location_type,direction,position''')

        transfers_in = self.normalize(z_in.read('transfers.txt'))
        transfers_out = self.normalize(z_out.read('transfers.txt'))
        self.assertEqual(transfers_in, transfers_out)

        trips_in = self.normalize(z_in.read('trips.txt'))
        trips_out = self.normalize(z_out.read('trips.txt'))
        # Drops unused trip_type
        self.assertEqual(trips_in, b'''\
route_id,service_id,trip_id,direction_id,block_id,shape_id,trip_type
34,W.411,5215038,0,3401,235511,
''')
        self.assertEqual(trips_out, b'''\
route_id,service_id,trip_id,direction_id,block_id,shape_id
34,W.411,5215038,0,3401,235511
''')



"""
Django settings for GTFS Explorer

First generated using Django 1.6.1
Quick-start development settings - unsuitable for production
"""

from decouple import config, Csv
import dj_database_url
import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(__file__))

# SECURITY WARNING: keep the secret key used in production secret!
_default_secret_key = 'l$hc*-biy7*#7cic5q5r^mtf-2&l34pq4k7znn%)si+$h(i%e&'
SECRET_KEY = config('SECRET_KEY', default=_default_secret_key, cast=str)

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = config('DEBUG', default=True, cast=bool)
ALLOWED_HOSTS = config('ALLOWED_HOSTS', default='', cast=Csv())

# Application definition
INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.gis',
    'exploreapp',
    'multigtfs',
]

LOCAL_INSTALLED_APPS = list(config('LOCAL_INSTALLED_APPS',
                                   default='', cast=Csv()))
INSTALLED_APPS.extend(LOCAL_INSTALLED_APPS)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'exploreproj.urls'

WSGI_APPLICATION = 'exploreproj.wsgi.application'

# Database
_default_db = 'spatialite:///%s' % os.path.join(BASE_DIR, 'db.sqlite3')
DATABASES = {'default': config('DATABASE_URL',
                               default=_default_db,
                               cast=dj_database_url.parse)}

# Templates
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# Internationalization
LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True

# Static files (CSS, JavaScript, Images)
STATIC_URL = '/static/'

# Test runner
_env_test_runner = config('TEST_RUNNER', default='', cast=str)
if _env_test_runner:
    TEST_RUNNER = _env_test_runner



#!/usr/bin/env python
#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from setuptools import setup, find_packages
from setuptools.command.test import test
import sys
# Get the version from __init__.py
from multigtfs import __version__


class my_test(test):
    def run(self):
        import run_tests
        run_tests.main()


def get_long_description():
    import codecs
    with codecs.open('README.rst', 'r', 'utf-8') as f:
        readme = f.read()

    body_tag = ".. Omit badges from docs"
    readme_body_start = readme.index(body_tag)
    assert readme_body_start
    readme_body = readme[readme_body_start + len(body_tag):]

    with codecs.open('CHANGELOG.rst', 'r', 'utf-8') as f:
        changelog = f.read()
    old_tag = ".. Omit older changes from package"
    changelog_body_end = changelog.index(old_tag)
    assert changelog_body_end
    changelog_body = changelog[:changelog_body_end]

    long_description = """
%(readme_body)s

%(changelog_body)s

Older changes can be found in the `full documention`_.

.. _`full documention`: \
http://multigtfs.readthedocs.io/en/latest/changelog.html
""" % locals()
    return long_description


# Handle Py2/Py3 issue
if sys.version_info > (3, 0):
    # In Py3, package data is dict w/ text key
    package_data = {'multigtfs': ['tests/fixtures/*.zip']}
else:
    # In Py2, package data is dict w/ binary string
    package_data = {b'multigtfs': ['tests/fixtures/*.zip']}


setup(
    name='multigtfs',
    version=__version__,
    description='General Transit Feed Specification (GTFS) as a Django app',
    long_description=get_long_description(),
    author='John Whitlock',
    author_email='John-Whitlock@ieee.org',
    license='Apache License 2.0',
    url='https://github.com/tulsawebdevs/django-multi-gtfs',
    packages=find_packages(),
    install_requires=['Django>=1.8', 'jsonfield>=0.9.20'],
    keywords=['django', 'gtfs'],
    test_suite="run_tests",  # Ignored, but makes pyroma happy
    cmdclass={'test': my_test},
    zip_safe=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    include_package_data=True,
    package_data=package_data
)



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from django.utils.encoding import python_2_unicode_compatible
from jsonfield import JSONField

from multigtfs.models.base import models, Base
from multigtfs.models.fields import SecondsField


@python_2_unicode_compatible
class Frequency(Base):
    """Description of a trip that repeats without fixed stop times"""
    trip = models.ForeignKey('Trip')
    start_time = SecondsField(
        help_text="Time that the service begins at the specified frequency")
    end_time = SecondsField(
        help_text="Time that the service ends at the specified frequency")
    headway_secs = models.IntegerField(
        help_text="Time in seconds before returning to same stop")
    exact_times = models.CharField(
        max_length=1, blank=True,
        choices=((0, 'Trips are not exactly scheduled'),
                 (1, 'Trips are exactly scheduled from start time')),
        help_text="Should frequency-based trips be exactly scheduled?")
    extra_data = JSONField(default={}, blank=True, null=True)

    def __str__(self):
        return str(self.trip)

    class Meta:
        db_table = 'frequency'
        app_label = 'multigtfs'
        verbose_name_plural = "frequencies"

    # For Base import/export
    _column_map = (
        ('trip_id', 'trip__trip_id'),
        ('start_time', 'start_time'),
        ('end_time', 'end_time'),
        ('headway_secs', 'headway_secs'),
        ('exact_times', 'exact_times'))
    _filename = 'frequencies.txt'
    _rel_to_feed = 'trip__route__feed'
    _unique_fields = ('trip_id', 'start_time')



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from django.utils.encoding import python_2_unicode_compatible
from jsonfield import JSONField

from multigtfs.models.base import models, Base


@python_2_unicode_compatible
class ServiceDate(Base):
    """Dates that a route is active.

    Implements calendar_dates.txt
    """
    service = models.ForeignKey('Service')
    date = models.DateField(
        help_text="Date that the service differs from the norm.")
    exception_type = models.IntegerField(
        default=1, choices=((1, 'Added'), (2, 'Removed')),
        help_text="Is service added or removed on this date?")
    extra_data = JSONField(default={}, blank=True, null=True)

    def __str__(self):
        return (
            "%d-%s %s %s" % (
                self.service.feed.id, self.service.service_id, self.date,
                'Added' if self.exception_type == 1 else 'Removed'))

    class Meta:
        db_table = 'service_date'
        app_label = 'multigtfs'

    # For Base import/export
    _column_map = (
        ('service_id', 'service__service_id'),
        ('date', 'date'),
        ('exception_type', 'exception_type'))
    _filename = 'calendar_dates.txt'
    _rel_to_feed = 'service__feed'
    _sort_order = ('date', 'exception_type')
    _unique_fields = ('service_id', 'date')



#
# Copyright 2012-2014 John Whitlock
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from django.contrib.gis.geos import MultiLineString
from django.utils.encoding import python_2_unicode_compatible
from jsonfield import JSONField

from multigtfs.models.base import models, Base


@python_2_unicode_compatible
class Route(Base):
    """A transit route

    Maps to route.txt in the GTFS feed.
    """

    feed = models.ForeignKey('Feed')
    route_id = models.CharField(
        max_length=255, db_index=True,
        help_text="Unique identifier for route.")
    agency = models.ForeignKey(
        'Agency', null=True, blank=True, help_text="Agency for this route.")
    short_name = models.CharField(
        max_length=63,
        help_text="Short name of the route")
    long_name = models.CharField(
        max_length=255,
        help_text="Long name of the route")
    desc = models.TextField(
        "description",
        blank=True,
        help_text="Long description of a route")
    rtype = models.IntegerField(
        "route type",
        choices=((0, 'Tram, Streetcar, or Light rail'),
                 (1, 'Subway or Metro'),
                 (2, 'Rail'),
                 (3, 'Bus'),
                 (4, 'Ferry'),
                 (5, 'Cable car'),
                 (6, 'Gondola or Suspended cable car'),
                 (7, 'Funicular')),
        help_text='Type of transportation used on route')
    url = models.URLField(
        blank=True, help_text="Web page about for the route")
    color = models.CharField(
        max_length=6, blank=True,
        help_text="Color of route in hex")
    text_color = models.CharField(
        max_length=6, blank=True,
        help_text="Color of route text in hex")
    geometry = models.MultiLineStringField(
        null=True, blank=True,
        help_text='Geometry cache of Trips')
    extra_data = JSONField(default={}, blank=True, null=True)

    def update_geometry(self):
        """Update the geometry from the Trips"""
        original = self.geometry
        trips = self.trip_set.exclude(geometry=None)
        unique_coords = set()
        unique_geom = list()
        for t in trips:
            coords = t.geometry.coords
            if coords not in unique_coords:
                unique_coords.add(coords)
                unique_geom.append(t.geometry)
        self.geometry = MultiLineString(unique_geom)
        if self.geometry != original:
            self.save()

    def __str__(self):
        return "%d-%s" % (self.feed.id, self.route_id)

    class Meta:
        db_table = 'route'
        app_label = 'multigtfs'

    _column_map = (
        ('route_id', 'route_id'),
        ('agency_id', 'agency__agency_id'),
        ('route_short_name', 'short_name'),
        ('route_long_name', 'long_name'),
        ('route_desc', 'desc'),
        ('route_type', 'rtype'),
        ('route_url', 'url'),
        ('route_color', 'color'),
        ('route_text_color', 'text_color')
    )
    _filename = 'routes.txt'
    _sort_order = ('route_id', 'short_name')
    _unique_fields = ('route_id',)
